#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <R.h>
#include <Rmath.h>
#define M_PI 3.1415926
#define MATHLIB_STANDALONE 1
#define xmalloc(nbytes) malloc_or_exit(nbytes, __FILE__,__LINE__,__func__)
typedef struct numstruct {
	double numval;
	int logind;  /*0: not log(zero), 1: log(zero)*/
} numstruct;

void *malloc_or_exit(size_t nbytes, const char *file, int line, const char *func)
{
	void *p = malloc(nbytes);
	if (p==NULL) {
		fprintf(stderr, "%s, line %d, %s():"
				"unable to allocate %u bytes, calling exit()\n",
				file, line, func, nbytes);
		exit(EXIT_FAILURE); 
	}
	return p;
}

double *dvector(int n)
{
	return xmalloc(n * sizeof(double));
}

void free_dvector(double *v)
{
	free(v);
}

int *dintvector(int n)
{
	return xmalloc(n * sizeof(int));
}

void free_dintvector(int *v)
{
	free(v);
}

numstruct *dstrvector(int n)
{
	return xmalloc(n * sizeof(numstruct));
}

void free_dstrvector(numstruct *v)
{
	free(v);
}

double **dpvector(int n)
{
	return xmalloc(n * sizeof (double*));
}

void free_dpvector(double **v)
{
	free(*v);
}

numstruct **dpstrvector(int n)
{
	return xmalloc(n * sizeof (numstruct*));
}

void free_dpstrvector(numstruct **v)
{
	free(*v);
}

double **dmatrix(int rows, int cols)
{
	double **a = dpvector(rows+1);
	int i;
	for (i=0; i<rows; i++)
		a[i] = dvector(cols);
	a[rows] = NULL;
	return a;
}

void free_dmatrix(double **a)
{
	int i;
	for (i=0; a[i]!=NULL; i++)
		free(a[i]);
	free(a);
}

numstruct **dstrmatrix(int rows, int cols)
{
	numstruct **a = dpstrvector(rows+1);
	int i;
	for (i=0; i<rows; i++)
		a[i] = dstrvector(cols);
	a[rows] = NULL;
	return a;
}

void free_dstrmatrix(numstruct **a)
{
	int i;
	for (i=0; a[i]!=NULL; i++)
		free(a[i]);
	free(a);
}

double ***darray(int rows, int cols, int depths)
{
	double ***a = xmalloc((depths+1) * sizeof (double**));
	int i;
	for (i=0; i<depths; i++)
		a[i] = dmatrix(rows, cols);  
	a[depths] = NULL;
	return a;
}

void free_darray(double ***a)
{
	int i;
	for (i=0; a[i]!=NULL; i++)
		free_dmatrix(a[i]);
	free(a);
}

numstruct ***dstrarray(int rows, int cols, int depths)
{
	/*a[k][i][j] is the element in the ith row, jth col and kth layer.*/
	numstruct ***a = xmalloc((depths+1) * sizeof (numstruct**));
	int i;
	for (i=0; i<depths; i++)
		a[i] = dstrmatrix(rows, cols);  
	a[depths] = NULL;
	return a;
}

void free_dstrarray(numstruct ***a)
{
	int i;
	for (i=0; a[i]!=NULL; i++)
		free_dstrmatrix(a[i]);
	free(a);
}

double sum_vec(int n, double *a_vec)
{
	int i;
	double sum_result=0.;
	
	for (i=0; i<n; i++)
	{sum_result=sum_result+a_vec[i];}
	
	return sum_result;
}

double a_multi_b(int n, double *a_vec, double *b_vec)
{
	int i;
	double multi_result=0.;
	
	for (i=0; i<n; i++)
	{multi_result=multi_result+a_vec[i]*b_vec[i];}
	
	return multi_result;
}

double a_multi_b_mat(int n1, int n2, int n3, double **a_mat, double **b_mat, double **res_mat)
{/*a_mat is a n1*n2 matrix, b_mat is a n2*n3 matrix*/
	int i, j, k;
	
	for (i=0; i<n1; i++)
	{
		for (j=0; j<n3; j++)
		{
			res_mat[i][j]=0;
			for (k=0; k<n2; k++)
			{
				res_mat[i][j]=res_mat[i][j]+a_mat[i][k]*b_mat[k][j];
			}
		}
	}
	return 0;
}

double lgamma(double x)
{
    double x0,x2,xp,gl,gl0;
    int n,k;
    static double a[] = {
        8.333333333333333e-02,
		-2.777777777777778e-03,
        7.936507936507937e-04,
		-5.952380952380952e-04,
        8.417508417508418e-04,
		-1.917526917526918e-03,
        6.410256410256410e-03,
		-2.955065359477124e-02,
        1.796443723688307e-01,
	-1.39243221690590};
    
    x0 = x;
    if (x <= 0.0) return 1e308;
    else if ((x == 1.0) || (x == 2.0)) return 0.0;
    else if (x <= 7.0) {
        n = (int)(7-x);
        x0 = x+n;
    }
    x2 = 1.0/(x0*x0);
    xp = 2.0*M_PI;
    gl0 = a[9];
    for (k=8;k>=0;k--) {
        gl0 = gl0*x2 + a[k];
    }
    gl = gl0/x0+0.5*log(xp)+(x0-0.5)*log(x0)-x0;
    if (x <= 7.0) {
        for (k=1;k<=n;k++) {
            gl -= log(x0-1.0);
            x0 -= 1.0;
        }
    }
    return gl;
}

int factorial( int n )
{
    if ( n <= 1 )
        return 1;
    else
        return  n * factorial( n-1 );
}

double factsum(int n)
{/* this function returns log of factorial of a integral n. */
	if (n<=1)
		return 0;
	else
	    return log(n) + factsum(n-1);
}

double max_array(double *a_vec, int num_elements)
{
	int i;
	double max=-32000;
	for (i=0; i<num_elements; i++)
	{
		if (a_vec[i]>max)
		{
			max=a_vec[i];
		}
	}
	return(max);
}

double max_2Darray(double **a_vec, int num_rows, int num_cols)
{
	int i, j;
	double max=-32000;
	for (i=0; i<num_rows; i++)
	{
		for (j=0; j<num_cols; j++)
		{
		    if (a_vec[i][j]>max)
		    {
			    max=a_vec[i][j];
		    }
		}
	}
	return(max);
}

int max_which(double *a_vec, int num_elements)
{
	int i, i_max=0;
	double max=-32000;
	for (i=0; i<num_elements; i++)
	{
		if (a_vec[i]>max)
		{
			max=a_vec[i];
			i_max=i;
		}
	}
	return(i_max);
}

numstruct eln(double x)
{
	numstruct elnx;
	
	if (x == 0) 
    {  
		elnx.numval=0.1;
		elnx.logind=1;
    }
    else 
    {
        elnx.numval=log(x);
	    elnx.logind=0;
    }
    return elnx;
}

double eexp(numstruct x)
{
	double eexpx;
	
	if (x.logind==1)
    {eexpx=0;}
	else 
    {eexpx=exp(x.numval);}
	
	return eexpx;
}

numstruct elnsum(numstruct x, numstruct y)
{
	numstruct elnsumxy;
    numstruct num1;
	
	if (x.logind==1 && y.logind==1)
	{
		elnsumxy.numval=0.1;
		elnsumxy.logind=1;
	}
	if (x.logind==1 && y.logind==0)
	{
		elnsumxy.numval=y.numval;
		elnsumxy.logind=0;
	}
	if (x.logind==0 && y.logind==1)
	{
		elnsumxy.numval=x.numval;
		elnsumxy.logind=0;
	}
	if (x.logind==0 && y.logind==0)
	{
		if (x.numval>y.numval)
		{
			num1=eln(1+exp(y.numval-x.numval));
			elnsumxy.numval=x.numval+num1.numval;
			elnsumxy.logind=0;
		}
		else
		{
			num1=eln(1+exp(x.numval-y.numval));
			elnsumxy.numval=y.numval+num1.numval;
			elnsumxy.logind=0;
		}
	}
	return elnsumxy;
}

numstruct elnadd(numstruct x, double y)
{
	/*This function returns x+y, x might be LOGZERO*/
    numstruct elnaddxy;
	
	if (x.logind==1)
	{
		elnaddxy.numval=0.1;
		elnaddxy.logind=1;
	}
	else
	{
		elnaddxy.numval=x.numval+y;
		elnaddxy.logind=0;
	}
	
	return elnaddxy;
}

numstruct elntimes(numstruct x, double y)
{
	/*This function returns x*y, x might be LOGZERO*/
    numstruct elntimesxy;
	
	if (y==0)
	{
		elntimesxy.numval=0;
		elntimesxy.logind=0;
	}
	if (x.logind==1 && y!=0)
	{
		elntimesxy.numval=0.1;
		elntimesxy.logind=1;
	}
	if (x.logind!=1 && y!=0)
	{
		elntimesxy.numval=x.numval*y;
		elntimesxy.logind=0;
	}
	
	return elntimesxy;
}


numstruct elnproduct(numstruct x, numstruct y)
{
	numstruct elnproductxy;
	
	if (x.logind==1 || y.logind==1)
	{
		elnproductxy.numval=0.1;
		elnproductxy.logind=1;
	}
	else
	{
		elnproductxy.numval=x.numval+y.numval;
		elnproductxy.logind=0;
	}
	return elnproductxy;
}

numstruct elnpower(double x, double y)
{
	numstruct elnpowerxy;
	
	if (y==0)
    {
		elnpowerxy.numval=0;
		elnpowerxy.logind=0;
	}
	if (x==0 && y!=0)
    {
		elnpowerxy.numval=0.1;
		elnpowerxy.logind=1;
	}
	if (x!=0 && y!=0)
    {
	    elnpowerxy.numval=y*log(x);
		elnpowerxy.logind=0;
	}
	
	return elnpowerxy;
}

int elnfwd_mat_func(int T, int M, double *pi_vec, numstruct **eln_emis_mat, double ***trans_mat, numstruct **elnf_mat)
{
	int k, t, l;
	numstruct logalpha, num1, num2;
	
	for (k=0; k<M; k++)
    {
		elnf_mat[0][k].numval=0.1;
		elnf_mat[0][k].logind=1;
	}
	elnf_mat[0][1].numval=0.;
	elnf_mat[0][1].logind=0;
	for (t=1; t<T; t++) 
	{
	    for (k=0; k<M; k++)
	    { 
			logalpha.numval=0.1;
			logalpha.logind=1;
			for (l=0; l<M; l++)
			{ 
				num1=eln(trans_mat[t-1][l][k]);
				num2=elnproduct(elnf_mat[t-1][l],num1);
				logalpha=elnsum(logalpha, num2);
			}
			elnf_mat[t][k]=elnproduct(logalpha, eln_emis_mat[t][k]);
	    }
	}
	return 0;
}

int elnbwd_mat_func(int T, int M, double *pi_vec, numstruct **eln_emis_mat, double ***trans_mat, numstruct **elnb_mat)
{
	int k, t, l;
	numstruct logbeta, num2, num3, num4;
	
	for (k=0; k<M; k++)
	{
		elnb_mat[T-1][k].numval=0;
		elnb_mat[T-1][k].logind=0;
	}
	for (t=T-2; t>(-1); t--)
	{
		for (k=0; k<M; k++)
		{
			logbeta.numval=0.1;
			logbeta.logind=1;
		    for (l=0; l<M; l++)
		    {
				num2=elnproduct(eln_emis_mat[t+1][l], elnb_mat[t+1][l]);
				num3=eln(trans_mat[t][k][l]);
				num4=elnproduct(num3, num2);
				logbeta=elnsum(logbeta, num4);
		    }
		    elnb_mat[t][k]=logbeta;
		}
	}
	return 0;
}

int elnP_k_mat_func(int T, int M, numstruct **elnf_mat, numstruct **elnb_mat, numstruct **elnP_k)
{
	int t, k;
	numstruct normalizer, num1;
	
	for (t=0; t<T; t++)
	{
		normalizer.numval=0.1;
		normalizer.logind=1;
		for (k=0; k<M; k++)
		{
			elnP_k[t][k]=elnproduct(elnf_mat[t][k], elnb_mat[t][k]);
			normalizer=elnsum(normalizer, elnP_k[t][k]);
		}
		for (k=0; k<M; k++)
		{   
			num1.numval=-normalizer.numval;
			num1.logind=normalizer.logind;
			elnP_k[t][k]=elnproduct(elnP_k[t][k], num1);
		}
	}
	return 0;
}

int elnP_klt_mat_func(int T, int M, numstruct **elnf_mat, numstruct **elnb_mat, double ***trans_mat, numstruct **eln_emis_mat, numstruct ***elnP_klt)
{
	int t, k, l;
	numstruct normalizer, num1, num3, num4, num5;
	
	for (t=0; t<(T-1); t++)
	{
		normalizer.numval=0.1;
		normalizer.logind=1;
		for (k=0; k<M; k++)
		{
			for (l=0; l<M; l++)
			{
				num1 = eln(trans_mat[t][k][l]);
				num3 = elnproduct(eln_emis_mat[t+1][l], elnb_mat[t+1][l]);
				num4 = elnproduct(num1, num3);
				elnP_klt[t][k][l] = elnproduct(elnf_mat[t][k], num4);
				normalizer = elnsum(normalizer, elnP_klt[t][k][l]);
			}
		}
		for (k=0; k<M; k++)
		{
			for (l=0; l<M; l++)
			{
				num5.numval = -normalizer.numval;
				num5.logind = normalizer.logind;
				elnP_klt[t][k][l] = elnproduct(elnP_klt[t][k][l], num5);
			}
		}
	}
	return 0;
}

double trans_mat_func(int M, int T, double r, double **p_mat, double *posi_vec, double ***trans_mat)
{
	int t, k, l;
	double *p_sum=dvector(M), num1;
	
	for (t=0; t<(T-1); t++)
	{
		num1=1-exp(-r*(posi_vec[t+1]-posi_vec[t]));
		for (k=0; k<M; k++)
		{
			for (l=0; l<M; l++)
		    {trans_mat[t][k][l]=p_mat[k][l]*num1;}
		}
		for (k=0; k<M; k++)
		{
			p_sum[k]=sum_vec(M, p_mat[k]);
			trans_mat[t][k][k]=1-p_sum[k]*num1;
	    }
	}
	
	free_dvector(p_sum);
	return 0;
}

numstruct eln_emis_prob_func(int T, int M, double alpha, double beta, int *sample_vec, int *control_vec, double *K_vec, double c0, numstruct **eln_emis_mat)
{
	double *vec1=dvector(M), **MAT_a=dmatrix(M, T), *MAT_b=dvector(T), *MAT_c=dvector(M), **MAT_1=dmatrix(T, M), **MAT_2b=dmatrix(T,M);
	numstruct **MAT_2a=dstrmatrix(T,M), **MAT_2=dstrmatrix(T, M);
	numstruct ret;
    int *data_vec=dintvector(T);
	int t, k;
	
	for (t=0; t<T; t++)
	{data_vec[t]=sample_vec[t]+control_vec[t];}
	
	for (k=0; k<M; k++)
	{vec1[k]=log(K_vec[k]*c0+1+beta);}
	
	for (k=0; k<M; k++)
	{
		for (t=0; t<T; t++)
			MAT_a[k][t]=vec1[k]*data_vec[t];		
	}
	
	for (t=0; t<T; t++)
	    MAT_b[t]=alpha*log(beta)+lgamma(data_vec[t]+alpha)-lgamma(alpha);
	
	for (k=0; k<M; k++)
	    MAT_c[k]=log(pow((K_vec[k]*c0 + 1 + beta), alpha)-pow(beta, alpha));
	
	for (k=0; k<M; k++)
	{
		for (t=0; t<T; t++)
		{
			MAT_2b[t][k]=(double)factsum(sample_vec[t])+(double)factsum(control_vec[t]);
			MAT_2a[t][k]=elnpower(K_vec[k]*c0, (double)sample_vec[t]);
			MAT_1[t][k]=MAT_b[t]-MAT_a[k][t]-MAT_c[k];
			MAT_2[t][k]=elnadd(MAT_2a[t][k], -MAT_2b[t][k]);
		}
	}
	
	for (k=0; k<M; k++)
	{
		for (t=0; t<T; t++)
		{eln_emis_mat[t][k]=elnadd(MAT_2[t][k], MAT_1[t][k]);}
	}
	
	free_dvector(vec1);
	free_dmatrix(MAT_a);
	free_dvector(MAT_b);
	free_dvector(MAT_c);
	free_dmatrix(MAT_1);
	free_dmatrix(MAT_2b);
	free_dstrmatrix(MAT_2);
    free_dstrmatrix(MAT_2a);
	free_dintvector(data_vec);
	
	return ret;
}

double p_find_llimit_func(int M, int T, double *posi_vec, int k, double r, double ***P_klt)
{
	double **P_klt_nok=dmatrix(M, T-1), *M11=dvector(M), M1, /*max_M11, max_M111,*/ llimit;
	int l, t;
	
    for (t=0; t<T-1; t++)
    {
		for (l=0; l<M; l++)
		{
			P_klt_nok[l][t]=P_klt[t][k][l];
		}
		P_klt_nok[k][t]=0;
	}
	
	for (l=0; l<M; l++)
	{M11[l]=sum_vec(T-1, P_klt_nok[l]);
	}
	M1=sum_vec(M, M11);
	
	/*	max_M11=max_array(M11, M);
	 if(M1 > max_M11)
	 {max_M111=M1;}
	 else 
	 {max_M111=max_M11;}
	 
	 if (max_M111>1)
	 {llimit=max_M111+1*pow(0.1,10);}
	 else {}
	 */
	llimit=M1+M1*pow(0.1,20);
	
	free_dmatrix(P_klt_nok);
	free_dvector(M11);
	
	return llimit;
}

double p_find_mat_func(int M, int T, double x, double *posi_vec, int k, double r, double ***P_klt, double lambda)
{
	int l, t;
	double **P_klt_k=dmatrix(M, T-1), **P_klt_0=dmatrix(M, (T-1)), *P_klt_timesum=dvector(M), **trans_P_klt_k=dmatrix((T-1), M), **P_klt_ktimestrans_P_klt_k=dmatrix((T-1), M), p_find_sum, *p_find_sum_vec=dvector(T-1);
	double sum_P_klt=0;
	numstruct **eln_trans_P_klt_k=dstrmatrix(T,M);
	
	for (t=0; t<T-1; t++)
	{
		for (l=0; l<M; l++)
		{
			P_klt_k[l][t]=P_klt[t][k][l];
			P_klt_0[l][t]=P_klt_k[l][t];
		}
		P_klt_0[k][t]=0;
	}
	
	for (l=0; l<M; l++)
	{
		P_klt_timesum[l]=sum_vec(T-1, P_klt_0[l]);
	}
	
	for (t=0; t<T-1; t++)
	{
		for(l=0; l<M; l++)
		{
			trans_P_klt_k[t][l]=P_klt_timesum[l]*(1-exp(-r*(posi_vec[t+1]-posi_vec[t])))/x;
		}
		trans_P_klt_k[t][k]=1-sum_vec(M, trans_P_klt_k[t]);
	}
	
	for (t=0; t<T-1; t++)
	{
		for (l=0; l<M; l++)
		{
			eln_trans_P_klt_k[t][l]=eln(trans_P_klt_k[t][l]);
		}
	}
	
	for (t=0; t<T-1; t++)
	{
		for (l=0; l<M; l++)
		{
			if (eln_trans_P_klt_k[t][l].logind==1)
			{
				eln_trans_P_klt_k[t][l].numval=0;
			}
		}
	}
	
	for (t=0; t<T-1; t++)
	{
		for(l=0; l<M; l++)
		{
			P_klt_ktimestrans_P_klt_k[t][l]=P_klt_k[l][t]*eln_trans_P_klt_k[t][l].numval;
		}
		p_find_sum_vec[t]=sum_vec(M, P_klt_ktimestrans_P_klt_k[t]);
	}
	p_find_sum=sum_vec(T-1, p_find_sum_vec);
	
	sum_P_klt = sum_vec(M, P_klt_timesum) * lambda / x;
	
	free_dmatrix(P_klt_k);
	free_dmatrix(P_klt_0);
	free_dvector(P_klt_timesum);
	free_dmatrix(trans_P_klt_k);
	free_dmatrix(P_klt_ktimestrans_P_klt_k);
	free_dvector(p_find_sum_vec);
	free_dstrmatrix(eln_trans_P_klt_k);
	
	return (-1)*p_find_sum + sum_P_klt;
	/*return (-1)*p_find_sum;*/
}

double optimize_p(int M, int T, double *posi_vec, int k, double r, double ***P_klt, double lambda)
{
	/* lambda is a tunning parameter in the penalty. */
	int i;
	double llimit, d, x1, x2, range_min, range_max;
	
	llimit=p_find_llimit_func(M, T, posi_vec, k, r, P_klt);
	range_min=llimit;
	range_max=llimit+(lambda+1)*pow(10,5);
	if (k==1)
	{
		range_max=range_max+pow(10, 10);
	}
	/*printf("range_min is %f, range_max is %f\n",range_min, range_max);*/
	d = 0.61803*(range_max - range_min);
	i=0;
	while (d >=0.001) {
		if (i<=3000){
			i=i+1;
		    x1 = range_min + d;
		    x2 = range_max - d;
			if (p_find_mat_func(M, T, x2, posi_vec, k, r, P_klt, lambda)>p_find_mat_func(M, T, x1, posi_vec, k, r, P_klt, lambda)) {    
				range_min = x2;
				x2 = x1;
			} else {              
				range_max = x1;
				x1 = x2;
			}
			d = 0.61803*(range_max - range_min);
		}
		else {
			printf("too many iterations in finding p's, cannot converge!");
		}
	}
	return x1;
}

double r_find_func(int M, int T, double r, double **p_mat, double *posi_vec, double ***P_klt)
{
	double ***trans_mat_r=darray(M, M, (T-1)), ***MATRIX=darray(M, M, (T-1)), sum_MATRIX;
	int t, k, l;
	
	trans_mat_func(M, T, r, p_mat, posi_vec, trans_mat_r);
	sum_MATRIX=0;
	for (t=0; t<(T-1); t++)
	{
		for (k=0; k<M; k++)
		{
			for (l=0; l<M; l++)
			{
				MATRIX[t][k][l]=P_klt[t][k][l]*log(trans_mat_r[t][k][l]);
				sum_MATRIX=sum_MATRIX+MATRIX[t][k][l];
			}
		}
	}
	
	free_darray(trans_mat_r);
	free_darray(MATRIX);
	return (-1)*sum_MATRIX;
}

double optimize_r(int M, int T, double **p_mat, double *posi_vec, double ***P_klt)
{
	int i;
	double d, x1, x2, range_min=0, range_max=1;
	
	d = 0.61803*(range_max - range_min);
	i=0;
	while (d >=0.001) {
		if (i<=3000){
			i=i+1;
		    x1 = range_min + d;
		    x2 = range_max - d;
			if (r_find_func(M, T, x2, p_mat, posi_vec, P_klt)>r_find_func(M, T, x1, p_mat, posi_vec, P_klt)) {    
				range_min = x2;
				x2 = x1;
			} else {              
				range_max = x1;
				x1 = x2;
			}
			d = 0.61803*(range_max - range_min);
		}
		else {
			printf("too many iterations in finding r, cannot converge!");
		}
	}
	return x1;
}

double alpha_beta_find_func(int M, int T, double alpha, double beta, int *sample_vec, int *control_vec, double *K_vec, double c0, double **P_k)
{
	int k, t;
	double power_emis_sum=0;
	numstruct **eln_emis_mat=dstrmatrix(T, M), **power_emis=dstrmatrix(T, M);
	
	eln_emis_prob_func(T, M, alpha, beta, sample_vec, control_vec, K_vec, c0, eln_emis_mat);
	for (t=0; t<T; t++)
	{
		for (k=0; k<M; k++)
		{
			power_emis[t][k] = elntimes(eln_emis_mat[t][k], P_k[t][k]);
			power_emis_sum=power_emis_sum+power_emis[t][k].numval;
		}
	}
	free_dstrmatrix(power_emis);
	free_dstrmatrix(eln_emis_mat);
	
	return (-1)*power_emis_sum;
}

double optimize_alpha(int M, int T, double beta, int *sample_vec, int *control_vec, double *K_vec, double c0, double **P_k)
{
	int i;
	double d, x1, x2, range_min=0, range_max=150;
	
	d = 0.61803*(range_max - range_min);
	i=0;
	while (d >=0.001) {
		if (i<=3000){
			i=i+1;
		    x1 = range_min + d;
		    x2 = range_max - d;
			if (alpha_beta_find_func(M, T, x2, beta, sample_vec, control_vec, K_vec, c0, P_k)>alpha_beta_find_func(M, T, x1, beta, sample_vec, control_vec, K_vec, c0, P_k)) 
			{    
				range_min = x2;
				x2 = x1;
			} else {              
				range_max = x1;
				x1 = x2;
			}
			d = 0.61803*(range_max - range_min);
		}
		else {
			printf("too many iterations in finding alpha, cannot converge!");
		}
	}
	return x1;
}

double optimize_beta(int M, int T, double alpha, int *sample_vec, int *control_vec, double *K_vec, double c0, double **P_k)
{
	int i;
	double d, x1, x2, range_min=0, range_max=100;
	
	d = 0.61803*(range_max - range_min);
	i=0;
	while (d >=0.001) {
		if (i<=3000){
			i=i+1;
		    x1 = range_min + d;
		    x2 = range_max - d;
			if (alpha_beta_find_func(M, T, alpha, x2, sample_vec, control_vec, K_vec, c0, P_k)>alpha_beta_find_func(M, T, alpha, x1, sample_vec, control_vec, K_vec, c0, P_k)) 
			{    
				range_min = x2;
				x2 = x1;
			} else {              
				range_max = x1;
				x1 = x2;
			}
			d = 0.61803*(range_max - range_min);
		}
		else {
			printf("too many iterations in finding beta, cannot converge!");
		}
	}
	return x1;
}

double lik_hood_func(int T, int M, int state, double alpha, double beta, int *sample_vec, int *control_vec, double *K_vec, double c0)
{
	int t;
	double log_lik=0.;
	/*FILE *fp = fopen("R_m_HMM_output.txt", "a");*/
	
    numstruct **eln_emis_prob_mat=dstrmatrix(T, M);
	eln_emis_prob_func(T, M, alpha, beta, sample_vec, control_vec, K_vec, c0, eln_emis_prob_mat);
	
	for (t=0; t<T; t++)
	{
		log_lik=log_lik+eln_emis_prob_mat[t][state].numval;
	}
	
	/*fprintf(fp, "eln_emis_prob_mat[t][state].numval is: \n");
	for (t=0; t<2; t++)
	{
	   if(state<2)
	   fprintf(fp, "%f  ", eln_emis_prob_mat[t][state].numval); 
	}
	fprintf(fp, "\n");*/
	
	free_dstrmatrix(eln_emis_prob_mat);
	/*fclose(fp);*/
	return (-1)*log_lik;
}


double optimize_lik_hood_alpha(int T, int M, int state, double beta, int *sample_vec, int *control_vec, double *K_vec, double c0)
{
	int i;
	double d, x1, x2, range_min=0, range_max=150;
	/*FILE *fp = fopen("R_m_HMM_output.txt", "a");*/
	
	d = 0.61803*(range_max - range_min);
	i=0;
	while (d >=0.001) /*while (d >=0.5)*/{
		if (i<=3000){
		/*if (i<=3){*/
			i=i+1;
		    x1 = range_min + d;
		    x2 = range_max - d;
			
			/*fprintf(fp, "lik_hood_func(T, M, state, x2, beta, sample_vec, control_vec, K_vec, c0) is: %f \n", lik_hood_func(T, M, state, x2, beta, sample_vec, control_vec, K_vec, c0));
			fprintf(fp, "lik_hood_func(T, M, state, x1, beta, sample_vec, control_vec, K_vec, c0) is: %f \n", lik_hood_func(T, M, state, x1, beta, sample_vec, control_vec, K_vec, c0));
*/
			
			if (lik_hood_func(T, M, state, x2, beta, sample_vec, control_vec, K_vec, c0)>lik_hood_func(T, M, state, x1, beta, sample_vec, control_vec, K_vec, c0)) 
			{    
				range_min = x2;
				x2 = x1;
			} else {              
				range_max = x1;
				x1 = x2;
			}
			d = 0.61803*(range_max - range_min);
		}
		else {
			printf("too many iterations in finding likhood_alpha, cannot converge!");
		}
	}
	
	/*fclose(fp);*/
	return x1;
}

double optimize_lik_hood_beta(int T, int M, int state, double alpha, int *sample_vec, int *control_vec, double *K_vec, double c0)
{
	int i;
	double d, x1, x2, range_min=0, range_max=100;
	
	d = 0.61803*(range_max - range_min);
	i=0;
	while (d >=0.001) {
		if (i<=3000){
			i=i+1;
		    x1 = range_min + d;
		    x2 = range_max - d;
			
			if (lik_hood_func(T, M, state, alpha, x2, sample_vec, control_vec, K_vec, c0)>lik_hood_func(T, M, state, alpha, x1, sample_vec, control_vec, K_vec, c0)) 
			{    
				range_min = x2;
				x2 = x1;
			} else {              
				range_max = x1;
				x1 = x2;
			}
			d = 0.61803*(range_max - range_min);
		}
		else {
			printf("too many iterations in finding likhood_beta, cannot converge!");
		}
	}
	return x1;
}
/*
int posi_simu_function(int T, double *posi_vec)
{
    int *posi_in=dintvector(T+1), i=0, j=0, p, *cut=dintvector(8);
	double mu1=5, mu2=30;
	
    cut[0]=3;cut[1]=5;cut[2]=8;cut[3]=13;cut[4]=18;cut[5]=24;cut[6]=28;cut[7]=30;
	
	while(j<(T+1))
	{
		p=0;
		if(j!=cut[i])
		{
			while (p==0)
				p=rpois (mu1);
		}
		else
		{
			while (p==0)
				p=rpois (mu2);
			i=i+1;
		}
		posi_in[j]=p;
		j=j+1;
	}
	
    for (i = 0; i < T; i++) 
    {
		posi_vec[i]=0;
		for (j=0; j<i+1; j++)
        {posi_vec[i]=posi_vec[i]+posi_in[j];}
    }
	
	free_dintvector(posi_in);
	free_dintvector(cut);
	return 0;
}

int states_simu_func(int T, int M, double **p_mat_simu, double r_simu, double *posi_vec_simu, int *states_vec_simu)
{
	int t, a;
	double ***trans_mat_simu=darray(M, M, T-1), rand, a1, a2, a3;
	
	trans_mat_func(M, T, r_simu, p_mat_simu, posi_vec_simu, trans_mat_simu);
	states_vec_simu[0]=1;	
	for (t=1; t<T; t++)
	{
		rand=runif(0,1);		
		a=states_vec_simu[t-1];
		a1=trans_mat_simu[t-1][a][0];
		a2=trans_mat_simu[t-1][a][0]+trans_mat_simu[t-1][a][1];
		a3=trans_mat_simu[t-1][a][0]+trans_mat_simu[t-1][a][1]+trans_mat_simu[t-1][a][2];
		if (rand>0 && rand<=a1)
		{states_vec_simu[t]=0;}
		if (rand>a1 && rand<=a2)
		{states_vec_simu[t]=1;}
		if (rand>a2 && rand<=a3)
		{states_vec_simu[t]=2;}
		if (rand>a3 && rand<=1)
		{states_vec_simu[t]=3;}
	}
	
	free_darray(trans_mat_simu);
	return 0;
}

int sample_control_vec_simu_func(int T, double alpha_simu, double beta_simu, double *K_vec, int *states_vec_simu, double c0_simu, int *sample_vec, int *control_vec)
{
	int i, c, s, a;
	double *K_simu=dvector(T);
	
	for (i=0; i<T; i++)
	{
		a=states_vec_simu[i];
		K_simu[i]=K_vec[a];
	}
	
	i=0;
	while(i<T)
	{
		c=rnbinom(alpha_simu, 1-1/(1+beta_simu));
		s=rnbinom(alpha_simu+c, 1-(K_simu[i]*c0_simu)/(K_simu[i]*c0_simu+1+beta_simu));
		if (c+s!=0)
		{
			control_vec[i]=c;
			sample_vec[i]=s;
			i=i+1;
		}
	}
	
	free_dvector(K_simu);
	return 0;
}
*/

void CNV_HMM_func_1(int *Min, int *Tin, double *alphain, double *betain, int *sample_vec, int *control_vec, double *sample_vec_double, double *control_vec_double, double *posi_vec, double *K_vec, double *c0in, int *states_guess, double *pi_vec, double *rin, double *p_mat_vec, double *lambda_vec,
				   int *ii_out, double *alpha_beta_diff_out, double *alpha_out, double *beta_out, double *r_out, const char **filename)
{
	
	/*
	 Input: 
	 M                  : number of states
	 T                  : number of positions
	 alpha              : initial alpha parameter in gamma distribution
	 beta               : initial beta parameter in gamma distribution
	 sample_vec         : sample vector with length T, integer value
	 control_vec        : control vector with length T, integer value
	 control_vec_double : same as control_vec, but in double variable version
	 posi_vec           : position vector with length T, integer value
	 K_vec              : a vector with length M, indicating the ratio between each states to neutral state
	 c0                 : initial a ratio between mean of neutral in sample vector and mean of control vector, sum_vec(T, sample_vec_double)/sum_vec(T, control_vec_double)
	 states_guess       : initial guess for each position, usually 1
	 pi_vec             : the robability of each state at position 1, (0,1,0,0)
	 r                  : initial value for r parameter in the transition matrix
	 p_mat              : initial value for a matrix of parameters used in the transition matrix
	 Output:
	 alpha_out              : final iteration value for alpha
	 beta_out               : final iteration value for beta
	 states_guess           : final guess for the states for each position after converge 
	 r_out                  : final value for r parameter
	 c0_out                 : final value for the ratio between mean of neutral in sample vector and mean of control vector
	 p_mat                  : final value for a matrix of parameters used in the transition matrix
	 ii_out                 : the number of iterations needed to converge
	 eps_state_out          : the number difference between the converged state guess and the last step one
	 alpha_beta_diff_out    : the difference between the converged alpha_beta value and the last step ones  
	 */
	int M = Min[0], T=Tin[0];
	double alpha = alphain[0], beta = betain[0], c0 = c0in[0], r = rin[0];
	
    int k, l, t, ii, jj, j, i0, state; 
	double ***trans_mat=darray(M, M, (T-1)), alpha_star, beta_star, alpha_beta_diff, eps_alpha_beta;
	int *states_guess_ud=dintvector(T);
	numstruct **elnf_mat=dstrmatrix(T,M), **elnb_mat=dstrmatrix(T,M), **elnP_k=dstrmatrix(T,M), ***elnP_klt=dstrarray(M, M, (T-1)), **eln_emis_mat=dstrmatrix(T, M);
	double ***P_klt=darray(M, M, (T-1)), *x_vec=dvector(M), **p_mat_ud=dmatrix(M, M), P_klt_row_sum;
	double r_ud, eps_p_r_pi, **p_mat_star=dmatrix(M, M), r_star, **p_mat_dist=dmatrix(M, M), p_mat_dist_max, **P_k=dmatrix(T, M);
	double eps_ab, alpha_ud, beta_ud;
	double p_mat_r_diff;
	double lambda;
	double **p_mat=dmatrix(M, M);
	numstruct **eln_emis_prob_mat=dstrmatrix(T, M); 
	FILE *fp = fopen(*filename, "a");
	
	i0=0;
	state=1;
	eps_ab=5;
	
    fprintf(fp, "sample_vec is: \n");
	for (t=0; t<30; t++)
	{
	   fprintf(fp, "%d ", sample_vec[t]);
	}
	fprintf(fp, "\n");
	
	fprintf(fp, "control_vec is: \n");
	for (t=0; t<30; t++)
	{
	   fprintf(fp, "%d ", control_vec[t]);
	}
	fprintf(fp, "\n");
	
	fprintf(fp, "alpha  is: %f \n", alpha);
    fprintf(fp, "beta  is: %f \n", beta);
	
	fprintf(fp, "p_mat is: \n");
	for (k=0; k<M; k++)
	{
		for (l=0; l<M; l++)
		{
		    p_mat[k][l]=p_mat_vec[M*l+k];
			fprintf(fp, "%f \n", p_mat[k][l]);
		}
	}
	
	fprintf(fp, "K_vec is: \n");
	for (k=0; k<M; k++)
	{
	   fprintf(fp, "%f \n", K_vec[k]);
	}
	
	fprintf(fp, "c0 is: %f \n", c0);
		
	eln_emis_prob_func(T, M, alpha, beta, sample_vec, control_vec, K_vec, c0, eln_emis_prob_mat);
	fprintf(fp, "eln_emis_prob_mat logind is: \n");
			for (t=0; t<30; t++)
			{
				for (k=0; k<M; k++)
				{
                    fprintf(fp, "%d ", eln_emis_prob_mat[t][k].logind);
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "\n");
			
			fprintf(fp, "eln_emis_prob_mat numval is: \n");
			for (t=0; t<30; t++)
			{
				for (k=0; k<M; k++)
				{
                    fprintf(fp, "%f ", eln_emis_prob_mat[t][k].numval);
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "\n");
	
	
	/*while (eps_ab>0.001 && i0<500)*/
	while (eps_ab>0.001 && i0<30)
	{
		i0=i0+1;
		alpha_ud=optimize_lik_hood_alpha(T, M, state, beta, sample_vec, control_vec, K_vec, c0);
		beta_ud=optimize_lik_hood_beta(T, M, state, alpha_ud, sample_vec, control_vec, K_vec, c0);
		fprintf(fp, "alpha_ud  is: %f \n", alpha_ud);
        fprintf(fp, "beta_ud  is: %f \n", beta_ud);
		
		if (fabs(alpha-alpha_ud)>fabs(beta-beta_ud))
		{ eps_ab=fabs(alpha-alpha_ud); }
		else
		{ eps_ab=fabs(beta-beta_ud);  }
		
		alpha=alpha_ud;
		beta=beta_ud;
	}
	
	fprintf(fp, "alpha starting value is: %f \n", alpha);
    fprintf(fp, "beta starting value is: %f \n", beta);
	
	ii=0;
	p_mat_r_diff=10;
	alpha_beta_diff=10;

	/*while ((p_mat_r_diff>0.01 || alpha_beta_diff>0.01) && ii<100)*/
	while ((p_mat_r_diff>0.01 || alpha_beta_diff>0.01) && ii<30)
	{
		ii=ii+1;
		fprintf(fp, "Iteration %d \n", ii);
		eln_emis_prob_func(T, M, alpha, beta, sample_vec, control_vec, K_vec, c0, eln_emis_mat);
		fprintf(fp, "eln_emis_mat logind is: \n");
			for (t=0; t<30; t++)
			{
				for (k=0; k<M; k++)
				{
                    fprintf(fp, "%d ", eln_emis_mat[t][k].logind);
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "\n");
			
			fprintf(fp, "eln_emis_mat numval is: \n");
			for (t=0; t<30; t++)
			{
				for (k=0; k<M; k++)
				{
                    fprintf(fp, "%f ", eln_emis_mat[t][k].numval);
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "\n");
			
		fprintf(fp, "p_mat_star is: \n");
		for (k=0; k<M; k++)
		{
			for (l=0; l<M; l++)
			{
				p_mat_star[k][l]=p_mat[k][l];
				fprintf(fp, " %f \n", p_mat_star[k][l]);
			}
		}
		
		r_star=r;
		eps_p_r_pi=0.02;
		jj=0;
		
		/*while(eps_p_r_pi > 0.01 && jj<500)*/
		while(eps_p_r_pi > 0.01 && jj<30)
		{
			jj=jj+1;
			fprintf(fp, "Sub-iteration is %d \n", jj);
			trans_mat_func(M, T, r_star, p_mat_star, posi_vec, trans_mat);
			fprintf(fp, "trans_mat is: \n");
			for (t=0; t<30; t++)
			{
				for (k=0; k<M; k++)
				{
					for (l=0; l<M; l++)
					{
						fprintf(fp, "%f ", trans_mat[t][k][l]);
					}
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "\n");
			
			elnfwd_mat_func(T, M, pi_vec, eln_emis_mat, trans_mat, elnf_mat);
			fprintf(fp, "elnf_mat logind is: \n");
			for (t=0; t<30; t++)
			{
				for (k=0; k<M; k++)
				{
                    fprintf(fp, "%d ", elnf_mat[t][k].logind);
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "\n");
			
			fprintf(fp, "elnf_mat numval is: \n");
			for (t=0; t<30; t++)
			{
				for (k=0; k<M; k++)
				{
                    fprintf(fp, "%f ", elnf_mat[t][k].numval);
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "\n");
			
			elnbwd_mat_func(T, M, pi_vec, eln_emis_mat, trans_mat, elnb_mat);
			fprintf(fp, "elnb_mat logind is: \n");
			for (t=0; t<30; t++)
			{
				for (k=0; k<M; k++)
				{
                    fprintf(fp, "%d ", elnb_mat[t][k].logind);
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "\n");
			
			fprintf(fp, "elnb_mat numval is: \n");
			for (t=0; t<30; t++)
			{
				for (k=0; k<M; k++)
				{
                    fprintf(fp, "%f ", elnb_mat[t][k].numval);
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "\n");
			
			elnP_k_mat_func(T, M, elnf_mat, elnb_mat, elnP_k);
			fprintf(fp, "elnP_k logind is: \n");
			for (t=0; t<30; t++)
			{
				for (k=0; k<M; k++)
				{	
                     fprintf(fp, "%d ", elnP_k[t][k].logind);
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "\n");
			
			fprintf(fp, "elnP_k numval is: \n");
			for (t=0; t<30; t++)
			{
				for (k=0; k<M; k++)
				{
                    fprintf(fp, "%f ", elnP_k[t][k].numval);
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "\n");
			
			elnP_klt_mat_func(T, M, elnf_mat, elnb_mat, trans_mat, eln_emis_mat, elnP_klt);
			/*fprintf(fp, "P_klt is: \n");*/
			for (t=0; t<(T-1); t++)
			{
				for (k=0; k<M; k++)
				{
					for (l=0; l<M; l++)
					{
						P_klt[t][k][l]=eexp(elnP_klt[t][k][l]);
						/*fprintf(fp, "%f ", P_klt[t][k][l]);*/
					}
				}
				/*fprintf(fp, "\n");*/
			}
			/*fprintf(fp, "\n");*/
			
			fprintf(fp, "P_klt_row_sum is: \n");
			for (k=0; k<M; k++)
			{
				lambda=lambda_vec[k];
				x_vec[k]=optimize_p(M, T, posi_vec, k, r_star, P_klt, lambda);
				for (l=0; l<M; l++)
				{
					P_klt_row_sum=0;
					for (t=0; t<(T-1); t++)
					    P_klt_row_sum=P_klt_row_sum+P_klt[t][k][l];
					p_mat_ud[k][l]=P_klt_row_sum/x_vec[k];
					fprintf(fp, "%f ", P_klt_row_sum);
				}
				p_mat_ud[k][k]=0;	
				fprintf(fp, "\n");
			}
			
			fprintf(fp, "x_vec is: ");
			for (k=0; k<M; k++)
			{
				fprintf(fp,"%f ", x_vec[k]);
		    }
			fprintf(fp, "\n");
			
			r_ud=optimize_r(M, T, p_mat_ud, posi_vec, P_klt);
			
			
			for (k=0; k<M; k++)
			{
				for (l=0; l<M; l++)
				{
					p_mat_dist[k][l]=fabs(p_mat_star[k][l]-p_mat_ud[k][l]);
				}
			}
			
			p_mat_dist_max=max_2Darray(p_mat_dist, M, M);
			
			if (p_mat_dist_max>fabs(r_ud-r_star))
			{eps_p_r_pi=p_mat_dist_max;}
			else {eps_p_r_pi=fabs(r_ud-r_star);}
			
			for (k=0; k<M; k++)
			{
				for (l=0; l<M; l++)
				{
					p_mat_star[k][l]=p_mat_ud[k][l];
				}
			}		
			r_star=r_ud;
			
			fprintf(fp, "eps_p_r_pi is: %f \n", eps_p_r_pi);
			fprintf(fp, "p_mat_star is: \n");
			
			for (k=0; k<M; k++)
			{
				for (l=0; l<M; l++)
				{
					fprintf(fp, "%e ", p_mat_star[k][l]);
				}
				fprintf(fp, "\n");
			}
			fprintf(fp, "r_star is: %f \n", r_star);
			fprintf(fp, "\n");
			
		}	
		
		for (k=0; k<M; k++)
		{
			for (l=0; l<M; l++)
			{
				p_mat_dist[k][l]=fabs(p_mat_star[k][l]-p_mat[k][l]);
			}
		}
		
		p_mat_dist_max=max_2Darray(p_mat_dist, M, M);
		
		if (p_mat_dist_max>fabs(r-r_star))
		{p_mat_r_diff=p_mat_dist_max;}
		else {p_mat_r_diff=fabs(r-r_star);}
		fprintf(fp, "p_mat_r_diff is: %f \n", p_mat_r_diff);
		
		for (k=0; k<M; k++)
		{
			for (l=0; l<M; l++)
			{
				p_mat[k][l]=p_mat_ud[k][l];
			}
		}	
		
		r=r_star;
		
		for (t=0; t<T; t++)
		{
			for (k=0; k<M; k++)
			{
				P_k[t][k]=eexp(elnP_k[t][k]);
			}
			states_guess_ud[t]=max_which(P_k[t], M);
		}
		for (t=0; t<T; t++)
		{
			states_guess[t]=states_guess_ud[t];
		}
		
		alpha_star=alpha;
		beta_star=beta;
		eps_alpha_beta=10;
		j=0;
		while(eps_alpha_beta > 0.01 && j<30)
		{
			j=j+1;
			alpha_ud=optimize_alpha(M, T, beta_star, sample_vec, control_vec, K_vec, c0, P_k);
			beta_ud=optimize_beta(M, T, alpha_ud, sample_vec, control_vec, K_vec, c0, P_k);
			
			if (fabs(alpha_star-alpha_ud)>fabs(beta_star-beta_ud))
			{ eps_alpha_beta=fabs(alpha_star-alpha_ud); }
			else
			{ eps_alpha_beta=fabs(beta_star-beta_ud);  }
			
			alpha_star=alpha_ud;
			beta_star=beta_ud;
		}
		
		if (fabs(alpha_star-alpha)>fabs(beta_star-beta))
		    alpha_beta_diff=fabs(alpha_star-alpha);
		else 
			alpha_beta_diff=fabs(beta_star-beta);
		
		alpha=alpha_star;
		beta=beta_star;
		
		fprintf(fp, "alpha_beta_diff %f \n", alpha_beta_diff);
		fprintf(fp, "alpha %f \n", alpha);
		fprintf(fp, "beta %f \n", beta);
		fprintf(fp, "r: %f \n", r);
		fprintf(fp, "states_guess: \n");
	    for (t=0; t<30; t++)
	    {
		    fprintf(fp, "%d ",states_guess[t]);			
	    }
		fprintf(fp, "\n");
		
	    fprintf(fp, "p_mat is: \n");
		for (k=0; k<M; k++)
		{
			for (l=0; l<M; l++)
			{
				fprintf(fp, "%e ", p_mat[k][l]);
			}
			fprintf(fp, "\n");
		}	
		fprintf(fp, "\n");
		fprintf(fp, "\n");
		fprintf(fp, "\n");
		
		
	}
	
	ii_out[0]=ii;
	alpha_beta_diff_out[0]=alpha_beta_diff;
	alpha_out[0]=alpha;
	beta_out[0]=beta;
	r_out[0]=r;
    printf("done. \n");	
	
	free_dvector(x_vec);
	free_dintvector(states_guess_ud);
	free_darray(trans_mat);
	free_darray(P_klt);	
	free_dstrmatrix(eln_emis_mat);
	free_dmatrix(p_mat_ud);
	free_dmatrix(p_mat_star);
	free_dmatrix(p_mat_dist);
	free_dmatrix(p_mat);
	free_dmatrix(P_k);		
	free_dstrmatrix(elnf_mat);
	free_dstrmatrix(elnb_mat);
	free_dstrmatrix(elnP_k);
	free_dstrarray(elnP_klt);
	
	free_dstrmatrix(eln_emis_prob_mat);
	
	fclose(fp);

	/*return 0;*/
}


/*
int main(void)
{
	int M=4, T=20744, t, k, l;
	double alpha, beta, c0, r;
	int *sample_vec=dintvector(T), *control_vec=dintvector(T), *posi_vec=dintvector(T), *states_guess=dintvector(T), *ii_out=dintvector(1);
	double *K_vec=dvector(M), *pi_vec=dvector(M), **p_mat=dmatrix(M, M), *control_vec_double=dvector(T), *sample_vec_double=dvector(T);
	double *alpha_beta_diff_out=dvector(1), *alpha_out=dvector(1), *beta_out=dvector(1), *r_out=dvector(1);
	double *lambda_vec=dvector(M);
	
	FILE *finp_sample;
	FILE *finp_control;
	FILE *finp_posi;
	FILE *fout_guess;
	
	K_vec[0]=0.5; K_vec[1]=1; K_vec[2]=2; K_vec[3]=0;
	pi_vec[0]=0; pi_vec[0]=1; pi_vec[2]=0; pi_vec[3]=0;
	lambda_vec[0]=pow(10,5); lambda_vec[1]=pow(10,10); lambda_vec[2]=pow(10,5); lambda_vec[3]=pow(10,5); 
	
	for (k=0; k<M; k++)
	{
		for (l=0; l<M; l++)
		{p_mat[k][l]=0.001;
		}
		p_mat[k][k]=0;
	}
	
	alpha=100;
	beta=2;
	r=0.5;
	
	
	finp_sample=fopen("simu_tot_Tcounts_chr4_combine_5.txt","r");
	finp_control=fopen("simu_tot_Ncounts_chr4_combine_5.txt","r");
	finp_posi=fopen("simu_tot_posi_chr4_combine_5.txt","r");
	fout_guess=fopen("states_guess_out_simu_chr4_5_5_10_5_5.txt","w");
	
	for (t=0; t<T; t++)
	{
		fscanf(finp_sample, "%d", &sample_vec[t]);
		fscanf(finp_control, "%d", &control_vec[t]);
		fscanf(finp_posi, "%d", &posi_vec[t]);
	}
	
	for (t=0; t<T; t++)
	{
		sample_vec_double[t]=(double)sample_vec[t];
		control_vec_double[t]=(double)control_vec[t];
	}
	
	c0=1.248872;    
	
	for (t=0; t<T; t++)
	{
		states_guess[t]=1;
	}
	

	printf("lambda vector is: \n");
    for (k=0; k<M; k++)
	{
		printf("%f ", lambda_vec[k]);
	}
	printf("\n");
	 
	CNV_HMM_func_1(M, T, alpha, beta, sample_vec, control_vec, sample_vec_double, control_vec_double, posi_vec, K_vec, c0, states_guess, pi_vec, r, p_mat, lambda_vec,
				   ii_out, alpha_beta_diff_out, alpha_out, beta_out, r_out);

	for (t=0; t<T; t++)
	{
		fprintf(fout_guess, "%d ", states_guess[t]);
		fprintf(fout_guess, "\n");
	}	
	
	free_dintvector(sample_vec);
	free_dintvector(control_vec);
	free_dintvector(posi_vec);
	free_dintvector(states_guess);
	free_dintvector(ii_out);
	free_dvector(K_vec);
	free_dvector(pi_vec);
	free_dvector(control_vec_double);
	free_dvector(sample_vec_double);
	free_dvector(alpha_beta_diff_out);
	free_dvector(r_out);
	free_dvector(alpha_out);
	free_dvector(beta_out);
	free_dvector(lambda_vec);
	free_dmatrix(p_mat);
	
	fclose(finp_sample);
	fclose(finp_control);
	fclose(finp_posi);
	fclose(fout_guess);
	
	return EXIT_SUCCESS;
}

*/






















